searchData={"items":[{"type":"module","title":"Prompt","doc":"Prompt provides a complete solution for building interactive command line applications.\n\nIt's very flexible and can be used just to provide helpers for taking input from the user and displaying output.","ref":"Prompt.html"},{"type":"module","title":"Basic Usage - Prompt","doc":"`import Prompt` includes utilities for:\n  * printing to the screen          -> `display/1`\n  * printing tables to the screen   -> `table/1`\n  * asking for confirmation         -> `confirm/1`\n  * picking from a list of choices  -> `select/2`\n  * asking for passwords            -> `password/1`\n  * free form text input            -> `text/1`","ref":"Prompt.html#module-basic-usage"},{"type":"module","title":"Advanced usage - Prompt","doc":"See `Prompt.Router`","ref":"Prompt.html#module-advanced-usage"},{"type":"module","title":"Building for Distribution - Prompt","doc":"There are a couple of different options for building a binary ready for distributing. Which ever approach you decide to use, you'll probably want to keep the docs instead of stripping them.\nFor escripts, you'll add the following  to the escript key in mix.exs, if using Bakeware, you'll add it to the releases key.\n\n```\n:strip_beams: [keep: [\"Docs\"]]\n```","ref":"Prompt.html#module-building-for-distribution"},{"type":"module","title":"Escript - Prompt","doc":"An [escript](https://hexdocs.pm/mix/master/Mix.Tasks.Escript.Build.html) is the most straightforward approach, but requires that erlang is already installed on the system.","ref":"Prompt.html#module-escript"},{"type":"module","title":"Bakeware - Prompt","doc":"This has been my preferred approach recently. [Bakeware](https://hexdocs.pm/bakeware/readme.html) uses releases to build a single executable binary that can be run on the system without the dependency on erlang or elixir.\n\nFor Bakeware, I also set `export RELEASE_DISTRIBUTION=none` in `rel/env.sh.eex` and `rel/env.bat.eex` - unless you need erlang distribution in your CLI.\n\nFor a complete example see [Genex](https://git.sr.ht/~ahappydeath/genex)\n\nRun `MIX_ENV=prod mix release` to build the binary. This will output the usual release messages for a `mix release` command, but in the case of a CLI, it's a bit of a red herring. The binary you want to run as a CLI will be in `_build/prod/rel/bakeware/ `.","ref":"Prompt.html#module-bakeware"},{"type":"module","title":"Burrito - Prompt","doc":"[Burrito](https://github.com/burrito-elixir/burrito) is another option for building a single binary artifact. It is unique because you can build binaries for windows, mac and linux all from one machine.\n\nFor a complete example see [Genex](https://github.com/silbermm/genex)\n\nRun `MIX_ENV=prod mix release` to build the binary. This will output the usual release messages for a `mix release` command, but in the case of a CLI, it's a bit of a red herring. The binary you want to run as a CLI will be in `burrito_out/ `.","ref":"Prompt.html#module-burrito"},{"type":"type","title":"Prompt.argv/0","doc":"The list of strings coming from the commmand-line arguments","ref":"Prompt.html#t:argv/0"},{"type":"function","title":"Prompt.choice/3","doc":"Display a choice prompt with custom answers.\n\nTakes a keyword list of answers in the form of atom to return and string to display.\n\n`[yes: \"y\", no: \"n\"]`\n\nwill show \"(y/n)\" and return `:yes` or `:no` based on the choice.\n\nSupported options:\n* `:color` - The text color. One of `[:black, :blue, :cyan, :green, :light_black, :light_blue, :light_cyan, :light_green, :light_magneta, :light_red, :light_white, :light_yellow, :magenta, :red, :white, :yellow]`.\n\n* `:background_color` - The background color. One of `[:black, :blue, :cyan, :green, :light_black, :light_blue, :light_cyan, :light_green, :light_magneta, :light_red, :light_white, :light_yellow, :magenta, :red, :white, :yellow]`.\n\n* `:default_answer` (`t:atom/0`) - The default answer for the choices. Defaults to the first choice.","ref":"Prompt.html#choice/3"},{"type":"function","title":"Examples - Prompt.choice/3","doc":"iex> Prompt.choice(\"Save password?\",\n    ...>   [yes: \"y\", no: \"n\", regenerate: \"r\"],\n    ...>   default_answer: :regenerate\n    ...> )\n    \"Save Password? (y/n/R):\" [enter]\n    iex> :regenerate","ref":"Prompt.html#choice/3-examples"},{"type":"type","title":"Prompt.command_list/0","doc":"A keyword list of commands and implementations of `Prompt.Command`","ref":"Prompt.html#t:command_list/0"},{"type":"type","title":"Examples - Prompt.command_list/0","doc":"iex> commands = [help: CLI.Commands.Help, version: CLI.Commands.Version]","ref":"Prompt.html#t:command_list/0-examples"},{"type":"function","title":"Prompt.confirm/2","doc":"Display a Y/n prompt.\n\nSets 'Y' as the the default answer, allowing the user to just press the enter key. To make 'n' the default answer pass the option `default_answer: :no`\n\nSupported options:\n* `:color` - The text color. One of `[:black, :blue, :cyan, :green, :light_black, :light_blue, :light_cyan, :light_green, :light_magneta, :light_red, :light_white, :light_yellow, :magenta, :red, :white, :yellow]`.\n\n* `:background_color` - The background color. One of `[:black, :blue, :cyan, :green, :light_black, :light_blue, :light_cyan, :light_green, :light_magneta, :light_red, :light_white, :light_yellow, :magenta, :red, :white, :yellow]`.\n\n* `:default_answer` - The default answer to the confirmation. The default value is `:yes`.\n\n* `:mask_line` (`t:boolean/0`) - If set to true, this will mask the current line by replacing it with `#####`. Useful when showing passwords in the terminal. The default value is `false`.","ref":"Prompt.html#confirm/2"},{"type":"function","title":"Examples - Prompt.confirm/2","doc":"iex> Prompt.confirm(\"Send the email?\")\n    \"Send the email? (Y/n):\" Y\n    iex> :yes\n\n    iex> Prompt.confirm(\"Send the email?\", default_answer: :no)\n    \"Send the email? (y/N):\" [enter]\n    iex> :no","ref":"Prompt.html#confirm/2-examples"},{"type":"function","title":"Prompt.display/2","doc":"Writes text to the screen.\n\nTakes a single string argument or a list of strings where each item in the list will be diplayed on a new line.\n\n\nSupported options:\n* `:color` - The text color. One of `[:black, :blue, :cyan, :green, :light_black, :light_blue, :light_cyan, :light_green, :light_magneta, :light_red, :light_white, :light_yellow, :magenta, :red, :white, :yellow]`. Defaults to the terminal default.\n\n* `:background_color` - The background color. One of `[:black, :blue, :cyan, :green, :light_black, :light_blue, :light_cyan, :light_green, :light_magneta, :light_red, :light_white, :light_yellow, :magenta, :red, :white, :yellow]`. Defaults to the terminal default.\n\n* `:position` - Print the content starting from the leftmost position or the rightmost position The default value is `:left`.\n\n* `:mask_line` (`t:boolean/0`) - If set to true, this will mask the current line by replacing it with `#####`. Useful when showing passwords in the terminal. The default value is `false`.","ref":"Prompt.html#display/2"},{"type":"function","title":"Examples - Prompt.display/2","doc":"iex> Prompt.display(\"Hello from the terminal!\")\n    \"Hello from the terminal!\"\n\n    iex> Prompt.display([\"Hello\", \"from\", \"the\", \"terminal\"])\n    \"Hello\"\n    \"from\"\n    \"the\"\n    \"terminal\"","ref":"Prompt.html#display/2-examples"},{"type":"function","title":"Prompt.password/2","doc":"Prompt the user for input, but conceal the users typing.\n\nSupported options:\n* `:color` - The text color. One of `[:black, :blue, :cyan, :green, :light_black, :light_blue, :light_cyan, :light_green, :light_magneta, :light_red, :light_white, :light_yellow, :magenta, :red, :white, :yellow]`. Defaults to the terminal default.\n\n* `:background_color` - The background color. One of `[:black, :blue, :cyan, :green, :light_black, :light_blue, :light_cyan, :light_green, :light_magneta, :light_red, :light_white, :light_yellow, :magenta, :red, :white, :yellow]`. Defaults to the terminal default.","ref":"Prompt.html#password/2"},{"type":"function","title":"Examples - Prompt.password/2","doc":"iex> Prompt.password(\"Enter your passsword\")\n    \"Enter your password:\"\n    iex> \"super_secret_passphrase\"","ref":"Prompt.html#password/2-examples"},{"type":"function","title":"Prompt.select/3","doc":"Displays options to the user denoted by numbers.\n\nAllows for a list of 2 tuples where the first value is what is displayed\nand the second value is what is returned to the caller.\n\nSupported options:\n* `:color` - The text color. One of `[:black, :blue, :cyan, :green, :light_black, :light_blue, :light_cyan, :light_green, :light_magneta, :light_red, :light_white, :light_yellow, :magenta, :red, :white, :yellow]`. Defaults to the terminal default.\n\n* `:background_color` - The background color. One of `[:black, :blue, :cyan, :green, :light_black, :light_blue, :light_cyan, :light_green, :light_magneta, :light_red, :light_white, :light_yellow, :magenta, :red, :white, :yellow]`. Defaults to the terminal default.\n\n* `:multi` (`t:boolean/0`) - Allows multiple selections from the options presented. The default value is `false`.","ref":"Prompt.html#select/3"},{"type":"function","title":"Examples - Prompt.select/3","doc":"iex> Prompt.select(\"Choose One\", [\"Choice A\", \"Choice B\"])\n    \"  [1] Choice A\"\n    \"  [2] Choice B\"\n    \"Choose One [1-2]:\" 1\n    iex> \"Choice A\"\n\n    iex> Prompt.select(\"Choose One\", [{\"Choice A\", 1000}, {\"Choice B\", 1001}])\n    \"  [1] Choice A\"\n    \"  [2] Choice B\"\n    \"Choose One [1-2]:\" 2\n    iex> 1001\n\n    iex> Prompt.select(\"Choose as many as you want\", [\"Choice A\", \"Choice B\"], multi: true)\n    \"  [1] Choice A\"\n    \"  [2] Choice B\"\n    \"Choose as many as you want [1-2]:\" 1 2\n    iex> [\"Choice A\", \"Choice B\"]","ref":"Prompt.html#select/3-examples"},{"type":"function","title":"Prompt.table/2","doc":"Print an ASCII table of data. Requires a list of lists as input.\n\nSupported options:\n* `:header` (`t:boolean/0`) - Use the first element as the header for the table. The default value is `false`.\n\n* `:border` - Determine how the border is displayed, one of :normal (default), :markdown, or :none The default value is `:normal`.\n\n* `:color` - The text color. One of `[:black, :blue, :cyan, :green, :light_black, :light_blue, :light_cyan, :light_green, :light_magneta, :light_red, :light_white, :light_yellow, :magenta, :red, :white, :yellow]`. Defaults to the terminal default.","ref":"Prompt.html#table/2"},{"type":"function","title":"Examples - Prompt.table/2","doc":"iex> Prompt.table([[\"Hello\", \"from\", \"the\", \"terminal!\"],[\"this\", \"is\", \"another\", \"row\"]])\n    \"\n     +-------+------+---------+----------+\n     | Hello | from | the     | terminal |\n     | this  | is   | another | row      |\n     +-------+------+---------+----------+\n    \"\n\n    iex> Prompt.table([[\"One\", \"Two\", \"Three\", \"Four\"], [\"Hello\", \"from\", \"the\", \"terminal!\"],[\"this\", \"is\", \"another\", \"row\"]], header: true)\n    \"\n     +-------+------+---------+----------+\n     | One   | Two  | Three   | Four     |\n     +-------+------+---------+----------+\n     | Hello | from | the     | terminal |\n     | this  | is   | another | row      |\n     +-------+------+---------+----------+\n    \"\n\n    iex> Prompt.table([[\"One\", \"Two\", \"Three\", \"Four\"], [\"Hello\", \"from\", \"the\", \"terminal!\"],[\"this\", \"is\", \"another\", \"row\"]], header: true, border: :markdown)\n    \"\n     | One   | Two  | Three   | Four     |\n     |-------|------|---------|----------|\n     | Hello | from | the     | terminal |\n     | this  | is   | another | row      |\n    \"\n\n    iex> Prompt.table([[\"Hello\", \"from\", \"the\", \"terminal!\"],[\"this\", \"is\", \"another\", \"row\"]], border: :none)\n    \"\n     Hello from the     terminal\n     this  is   another row\n    \"","ref":"Prompt.html#table/2-examples"},{"type":"function","title":"Prompt.table_data/2","doc":"Use this to get an iolist back of the table. Useful when you want an ascii `table/1` for\nother mediums like markdown files.","ref":"Prompt.html#table_data/2"},{"type":"function","title":"Prompt.text/2","doc":"Display text on the screen and wait for the users text imput.\n\nSupported options:\n* `:color` - The text color. One of `[:black, :blue, :cyan, :green, :light_black, :light_blue, :light_cyan, :light_green, :light_magneta, :light_red, :light_white, :light_yellow, :magenta, :red, :white, :yellow]`. Defaults to the terminal default.\n\n* `:background_color` - The background color. One of `[:black, :blue, :cyan, :green, :light_black, :light_blue, :light_cyan, :light_green, :light_magneta, :light_red, :light_white, :light_yellow, :magenta, :red, :white, :yellow]`. Defaults to the terminal default.\n\n* `:min` (`t:integer/0`) - The minimum charactors required for input The default value is `0`.\n\n* `:max` (`t:integer/0`) - The maximum charactors required for input The default value is `0`.","ref":"Prompt.html#text/2"},{"type":"function","title":"Examples - Prompt.text/2","doc":"iex> Prompt.text(\"Enter your email\")\n    \"Enter your email:\" t@t.com\n    iex> t@t.com","ref":"Prompt.html#text/2-examples"},{"type":"behaviour","title":"Prompt.Command","doc":"Helps users define and build command line commands.\n\nDefines the behaviour for a Command.\n\nWe expect `init/1` to be called with the command line options and get\nback a data structure that is passed to `process/1` which handles all\nof the side effects of the command itself.","ref":"Prompt.Command.html"},{"type":"behaviour","title":"Example - Prompt.Command","doc":"```\ndefmodule MyCommand do\n  @moduledoc \"MyCommand's help message help() is defined in the __using__ macro that prints this message if called\"\n\n  use Prompt.Command\n\n  @impl true\n  def init(_argv) do\n    # parse list of args to a struct if desired\n    %SomeStruct{list: true, help: false, directory: \"path/to/dir\"}\n  end\n\n  @impl true\n  def process(%{help: true}), do: help() # this help function is defined by default in the macro that prints the @moduledoc when called\n  def process(%{list: true, help: false, directory: dir}) do\n    display(File.ls!(dir))\n  end\n\nend\n```\n\nIf this is used in a release, `help()` won't print the @moduledoc correctly because releases strip documentation by default. For this to work correctly, tell the release to keep docs:\n```\nreleases: [\n  appname: [\n    strip_beams: [keep: [\"Docs\"]]\n  ]\n]\n```","ref":"Prompt.Command.html#module-example"},{"type":"callback","title":"Prompt.Command.help/0","doc":"Prints the help available for this command","ref":"Prompt.Command.html#c:help/0"},{"type":"callback","title":"Prompt.Command.init/1","doc":"Takes the options passed in via the command line and\ntramforms them into a struct that the process command can handle","ref":"Prompt.Command.html#c:init/1"},{"type":"callback","title":"Prompt.Command.process/1","doc":"Processes the command and does the things required","ref":"Prompt.Command.html#c:process/1"},{"type":"module","title":"Prompt.Position","doc":"Manipulate the position of the cursor and the display of previous text.","ref":"Prompt.Position.html"},{"type":"function","title":"Prompt.Position.clear_lines/1","doc":"Clears the content from the previous `number` of lines\nand resets the cursor.","ref":"Prompt.Position.html#clear_lines/1"},{"type":"function","title":"Prompt.Position.mask_line/1","doc":"Mask the content on the terminal `relative_line` above.\nBy default will clear the line and put `#######` in its place\nthen move the cursor back to the current line.","ref":"Prompt.Position.html#mask_line/1"},{"type":"behaviour","title":"Prompt.Router","doc":"Router for Prompt\n\nSimplifies defining commands, sub-commands and arguments.\n\nChoose the module responsible for taking the command line arguments and \n`use Prompt.Router, otp_app: :your_app` at the top.\n\nThen simply define your commands and arguments.\n\nExposes a main/1 function that is called with the command line args","ref":"Prompt.Router.html"},{"type":"behaviour","title":"Arguments - Prompt.Router","doc":"See `arg/3`","ref":"Prompt.Router.html#module-arguments"},{"type":"behaviour","title":"Example - Prompt.Router","doc":"```elixir\ndefmodule My.CLI do\n  use Prompt.Router, otp_app: :my_app\n\n  command :checkout, My.CheckoutCommand do\n    arg :help, :boolean\n    arg :branch, :string, short: :b, default: \"main\"\n  end\n\n  command \"\", My.DefaultCommand do\n    arg :info, :boolean\n  end\nend\n\ndefmodule My.CheckoutCommand do\n  use Prompt.Command\n\n  @impl true\n  def process(arguments) do\n    # arguments will be a map of the defined arguments and their values\n    # from the command line input\n    # If someone used the command and passed `--branch  feature/test`, then\n    # `argmuments would look like `%{help: false, branch: \"feature/test\"}`\n    display(\"checking out \" <> arguments.branch)\n  end\nend\n\ndefmodule My.DefaultCommand do\n  use Prompt.Command\n\n  @impl true\n  def init(arguments) do\n    # you can implement the `c:init/1` callback to transform\n    # the arguments before `c:process/1` is called if you want\n    arguments\n  end\n  \n  @impl true\n  def process(arguments) do\n    # arguments will have a key of `:leftover` for anything\n    # passed to the command that doesn't have a `arg` defined.\n    # IF someone called this with `--info --test something`, then then\n    # arguments will look like `%{info: true, leftover: [\"--test\", \"something\"]}`\n    display(\"showing info\")\n  end\nend\n```","ref":"Prompt.Router.html#module-example"},{"type":"macro","title":"Prompt.Router.arg/3","doc":"Defines the arguments of a command.","ref":"Prompt.Router.html#arg/3"},{"type":"macro","title":"Argument Name - Prompt.Router.arg/3","doc":"This indicates what the user will type as the option to the sub-command.\nFor example,\n```elixir\narg :print, :boolean\n```\nwould allow the user to type `$ your_command --print`","ref":"Prompt.Router.html#arg/3-argument-name"},{"type":"macro","title":"Options - Prompt.Router.arg/3","doc":"Available options are:\n  * default - a default value if the user doesn't use this option\n  * short   - an optional short argument option i.e `short: :h` would all the user to type `-h`","ref":"Prompt.Router.html#arg/3-options"},{"type":"macro","title":"Prompt.Router.command/2","doc":"","ref":"Prompt.Router.html#command/2"},{"type":"macro","title":"Prompt.Router.command/3","doc":"Name of the subcommand that is expectedaany()\n\nTakes an atom or string as the command name and a Prompt.Command module.","ref":"Prompt.Router.html#command/3"},{"type":"callback","title":"Prompt.Router.generate_completions/1","doc":"Called when the flag `--completions string` is passed on the command-line.\n\nThe default behaviour for this is to write the completion script to the\nscreen.\n\nzsh is supported out of the box. If other completion scripts are required,\nthis callback will need to be implemented.\n\nsee https://github.com/zsh-users/zsh-completions/blob/master/zsh-completions-howto.org\n\nOverrideable","ref":"Prompt.Router.html#c:generate_completions/1"},{"type":"callback","title":"Prompt.Router.handle_exit_value/1","doc":"This function is called after the main function is done.\n\nIt does it's best to handle any value returned from a command and turn \nit into an integer, 0 being a successful command and any non-zero being\nan error.\n\nOverrideable","ref":"Prompt.Router.html#c:handle_exit_value/1"},{"type":"callback","title":"Prompt.Router.help/0","doc":"Prints help to the screen when there is an error, or `--help` is passed as an argument. \n\nOverridable","ref":"Prompt.Router.html#c:help/0"},{"type":"callback","title":"Prompt.Router.help/1","doc":"Prints help to the screen when there is an error with a string indicating the error\n\nOverridable","ref":"Prompt.Router.html#c:help/1"},{"type":"callback","title":"Prompt.Router.main/1","doc":"The function responsible for filtering and calling the correct command \nmodule based on command line input","ref":"Prompt.Router.html#c:main/1"},{"type":"callback","title":"Prompt.Router.version/0","doc":"Prints the version from the projects mix.exs file\n\nOverridable","ref":"Prompt.Router.html#c:version/0"},{"type":"extras","title":"Introduction","doc":"![prompt_with_text_lighter](https://user-images.githubusercontent.com/42816/115971052-5772c380-a514-11eb-8b43-dd49e81467f5.png)\n\n[![Hex.pm](https://img.shields.io/hexpm/v/prompt?style=flat-square)](https://hexdocs.pm/prompt/Prompt.html#content)\n\n# Terminal UI Library for Elixir\n* [Motivation](#motivation)\n* [Installation](#installation)\n* [Basic Usage](#basic-usage)\n  * [Display text on the screen](#display-text-on-the-screen)\n  * [Ask the user for input](#ask-the-user-for-input)\n  * [Ask the user for a password](#ask-the-user-for-a-password)\n  * [Ask the user for confirmation](#ask-the-user-for-confirmation)\n  * [Custom confirmation choices](#custom-confirmation-choices)\n  * [List of selections](#list-of-selections)\n  * [Tables](#tables)\n* [Advanced Usage](#advanced-usage-with-subcommands)\n* [Example Application](#example)\n\n[![Run in Livebook](https://livebook.dev/badge/v1/black.svg)](https://livebook.dev/run?url=https%3A%2F%2Fgithub.com%2Fsilbermm%2Fprompt%2Fblob%2Fmain%2Fexample.livemd)","ref":"introduction.html"},{"type":"extras","title":"Motivation - Introduction","doc":"To create a really great development experience and API for Elixir developers that want to build commandline tools.","ref":"introduction.html#motivation"},{"type":"extras","title":"Installation - Introduction","doc":"Add `prompt` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:prompt, \"~> 0.10.0\"}\n  ]\nend\n```\n\n[Read the official documentation](https://hexdocs.pm/prompt/Prompt.html)","ref":"introduction.html#installation"},{"type":"extras","title":"Basic Usage - Introduction","doc":"All of the following commands take a keyword list of options for things like text color and positioning.","ref":"introduction.html#basic-usage"},{"type":"extras","title":"Display text on the screen - Introduction","doc":"[Prompt.display/2](https://hexdocs.pm/prompt/Prompt.html#display/2)\n```elixir\nPrompt.display(\"Hello, world!\")\n```","ref":"introduction.html#display-text-on-the-screen"},{"type":"extras","title":"Ask the user for input - Introduction","doc":"[Prompt.text/2](https://hexdocs.pm/prompt/Prompt.html#text/2)\nUseful for prompting the user to enter freeform text\n```elixir\nPrompt.text(\"Enter info here\")\n```\nWill display:\n```bash\n> Enter info here:\n```\nand wait for the user to enter text","ref":"introduction.html#ask-the-user-for-input"},{"type":"extras","title":"Ask the user for a password - Introduction","doc":"[Prompt.password/2](https://hexdocs.pm/prompt/Prompt.html#password/2)\nWhen you need to hide the input that the user types\n```elixir\nPrompt.password(\"Enter password\")\n```","ref":"introduction.html#ask-the-user-for-a-password"},{"type":"extras","title":"Ask the user for confirmation - Introduction","doc":"[Prompt.confirm/2](https://hexdocs.pm/prompt/Prompt.html#confirm/2)\n```elixir\nPrompt.confirm(\"Are you sure?\")\n```\nWill display:\n```bash\n> Are you sure? (Y/n):\n```\nand will allow the user to just press [enter] to confirm\n\nIf you'd prefer `n` to be the default pass the `default_answer` option\n```elixir\nPrompt.confirm(\"Are you sure?\", default_answer: :no)\n```\nReturns `:yes` or `:no` based on the answer","ref":"introduction.html#ask-the-user-for-confirmation"},{"type":"extras","title":"Custom confirmation choices - Introduction","doc":"[Prompt.choice/2](https://hexdocs.pm/prompt/Prompt.html#choice/2)\nSometimes yes/no aren't the only choices a user can make, this method allows you to pass any choices as the confirmation.\n```elixir\nPrompt.choice(\"Accept, Reload or Cancel\", accept: \"a\", reload: \"r\", cancel: \"c\")\n```\ndisplays\n```bash\n> Accept, Reload or Cancel (A/r/c):\n```\nReturns the key of the answer i.e `:accept`, `:reload` or `cancel` in this exammple","ref":"introduction.html#custom-confirmation-choices"},{"type":"extras","title":"List of selections - Introduction","doc":"[Prompt.select/2](https://hexdocs.pm/prompt/Prompt.html#select/2)\nTo show the user a list of options to select from\n\n```elixir\nPrompt.select(\"Choose a protocol\", [\"file://\", \"ssh://\", \"ftp://\"])\n```\nDisplays:\n```bash\n  [1] file://\n  [2] ssh://\n  [3] ftp://\n> Choose a protocol [1-3]:\n```\nand returns a string of their choice","ref":"introduction.html#list-of-selections"},{"type":"extras","title":"Tables - Introduction","doc":"[Prompt.table/2](https://hexdocs.pm/prompt/Prompt.html#table/2)\nTo show a table of data\n```elixir\nPrompt.table([[\"Hello\", \"from\", \"the\", \"terminal!\"],[\"this\", \"is\", \"another\", \"row\"]])\n```\nWill display\n```bash\n> +-------+------+---------+----------+\n  | Hello | from | the     | terminal |\n  | this  | is   | another | row      |\n  +-------+------+---------+----------+\n```","ref":"introduction.html#tables"},{"type":"extras","title":"Advanced Usage with Subcommands - Introduction","doc":"To use the more advanced features, see the [official documentation](https://hexdocs.pm/prompt/Prompt.html#module-subcommands)","ref":"introduction.html#advanced-usage-with-subcommands"},{"type":"extras","title":"Example - Introduction","doc":"For a complete example, take a look at [Genex - a password manager](https://git.sr.ht/~ahappydeath/genex)","ref":"introduction.html#example"},{"type":"extras","title":"Example","doc":"# Simple Example\n\n```elixir\nMix.install([\n  {:prompt, \"~> 0.9.3\"}\n])\n```","ref":"example.html"},{"type":"extras","title":"Starting Out - Example","doc":"Lets go through an example of creating and building a new commandline/terminal app with `Prompt` that will acknowledge a person.\nLets call it `ack`.\n\n<!-- livebook:{\"break_markdown\":true} -->\n\nLets define a `CLI` module and create where we can define what arguments we allow and what options are available for our tool.\n\n```elixir\ndefmodule CLI do\n  @moduledoc \"\"\"\n  DOCUMENTATION FOR THE TOOL\n  \"\"\"\n  use Prompt.Router, otp_app: :ack\n\n  command :hello, HelloCommand do\n    arg(:help, :boolean)\n  end\nend\n```","ref":"example.html#starting-out"},{"type":"extras","title":"Now Create the Command Handler - Example","doc":"Now we'll have to create the handler for the `hello` command.\n\n```elixir\ndefmodule HelloCommand do\n  @moduledoc \"\"\"\n  The help message\n\n  ack hello  \n\n    --help prints this help message\n\n  \"\"\"\n  use Prompt.Command\n\n  @impl true\n  def init(opts) do\n    # opts is map of data which includes the \n    # options for the command and any other data passed in\n    # Our opts here look like give the following incanatation `ack hello bob`:\n    # `%{help: false, leftover: [\"bob\"]}\n\n    # `init/1` can be used to transform any arguments if needed.\n    Map.put(opts, :name, List.first(opts.leftover, nil))\n  end\n\n  @impl true\n  def process(%{help: true}), do: help()\n\n  def process(%{name: name}) do\n    display(\"Hello #{name}\", color: :green)\n  end\nend\n```","ref":"example.html#now-create-the-command-handler"},{"type":"extras","title":"Simulate calling the CLI - Example","doc":"```elixir\nCLI.main([\"hello\", \"bob\"])\n```\n\n```elixir\nCLI.main([\"hello\", \"--help\"])\n```","ref":"example.html#simulate-calling-the-cli"},{"type":"extras","title":"Bundle and Distribute - Example","doc":"To build your app, you'll need to decide if you want an escript or a binary. The former will require\nthe user of your tool to have Elixir and Erlang installed on their system. A binary built using \nBakeware or Burrito can be distributed and used without having Elixir or Erlang on the system.\n\nSee the [documentation](https://hexdocs.pm/prompt/Prompt.html#module-building-for-distribution) for details on both","ref":"example.html#bundle-and-distribute"},{"type":"extras","title":"Changelog","doc":"# Changelog\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [0.10.1](https://codeberg.org/ahappydeath/prompt/releases/tag/v0.10.1) - 2025-01-31","ref":"changelog.html"},{"type":"extras","title":"Updated - Changelog","doc":"- Moved Repo to Codeberg from Sourcehut\n  - Dependencies including\n    - nimble_options (1.1.1)","ref":"changelog.html#updated"},{"type":"extras","title":"Added - Changelog","doc":"- Generated command `--complete` that will build a simple zsh completion script\n\n## [0.10.0]","ref":"changelog.html#added"},{"type":"extras","title":"Updated - Changelog","doc":"- Forward all params to fallback command instead of failing when unable to match a command.\n\n## [0.9.4]","ref":"changelog.html#updated"},{"type":"extras","title":"Updated - Changelog","doc":"- Documentation for building binary releases\n  - Livebook Example\n\n## [0.9.3]","ref":"changelog.html#updated"},{"type":"extras","title":"Added - Changelog","doc":"- Allow for border-less tables\n\n## [0.9.2] - 2023-07-31","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- a `short` option for arguments\n\n## [0.9.1] - 2023-07-17","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Properly handle the charlist return from `IO.read/2`\n\n## [0.9.0] - 2023-07-17","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- when a command is passed in that is not recognized, print help to the screen\n\n## [0.8.1] - 2022-12-13","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Better support for halting the runtime when a command is finished\n    - a new callback `handle_exit_value/1` that is overrideable and called after each command is done.\n\n## [0.8.0] - 2022-06-27","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Prompt.Router to simplify creating commands\n    - introduces the `command` and `arg` macros\n\n## [0.7.4] - 2022-06-06","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- min and max validations for text entry\n\n## [0.6.3] - 2021-11-28","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- use NimbleOptions to validate options\n  - background_color as an option for displaying text","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- modified the way to pass in text color\n    * takes an atom now instead of an `IO.ANSI`color function\n  - refactored `select/2` into it's own module\n  - refactored `choice/2` into it's own module\n  - refactored `confirm/2` into it's own module\n  - refactored `text/2` into it's own module\n\n\n## [0.6.2] - 2021-10-25","ref":"changelog.html#changed"},{"type":"extras","title":"Documentation - Changelog","doc":"- Added Livebook example as extra pages in hexdoc\n  - Organized `Prompt` functions as Input / Output\n  - Nested Modules for better readability\n  - Updated ex_docs\n  - Added GPL License file\n\n## [0.6.1] - 2021-10-21","ref":"changelog.html#documentation"},{"type":"extras","title":"Added - Changelog","doc":"- Many more tests\n  - Livebook notebook with an example\n  -\n\n## [0.6.0] - 2021-10-17","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- a `help/0` callback and the ability to override","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Instead of taking a list of tuples for commands (`[{\"command\", Module\"}]`), now take a Keyword list (`[command: Module]`)\n\n[0.8.1]: https://github.com/silbermm/prompt/releases/tag/v0.8.1\n[0.8.0]: https://github.com/silbermm/prompt/releases/tag/v0.8.0\n[0.7.4]: https://github.com/silbermm/prompt/releases/tag/v0.7.4\n[0.6.3]: https://github.com/silbermm/prompt/releases/tag/v0.6.3\n[0.6.2]: https://github.com/silbermm/prompt/releases/tag/v0.6.2\n[0.6.1]: https://github.com/silbermm/prompt/releases/tag/v0.6.1\n[0.6.0]: https://github.com/silbermm/prompt/releases/tag/v0.6.0","ref":"changelog.html#changed"},{"type":"extras","title":"License","doc":"GNU GENERAL PUBLIC LICENSE\n                       Version 3, 29 June 2007\n\n Copyright (C) 2007 Free Software Foundation, Inc.  \n Everyone is permitted to copy and distribute verbatim copies\n of this license document, but changing it is not allowed.\n\n                            Preamble\n\n  The GNU General Public License is a free, copyleft license for\nsoftware and other kinds of works.\n\n  The licenses for most software and other practical works are designed\nto take away your freedom to share and change the works.  By contrast,\nthe GNU General Public License is intended to guarantee your freedom to\nshare and change all versions of a program--to make sure it remains free\nsoftware for all its users.  We, the Free Software Foundation, use the\nGNU General Public License for most of our software; it applies also to\nany other work released this way by its authors.  You can apply it to\nyour programs, too.\n\n  When we speak of free software, we are referring to freedom, not\nprice.  Our General Public Licenses are designed to make sure that you\nhave the freedom to distribute copies of free software (and charge for\nthem if you wish), that you receive source code or can get it if you\nwant it, that you can change the software or use pieces of it in new\nfree programs, and that you know you can do these things.\n\n  To protect your rights, we need to prevent others from denying you\nthese rights or asking you to surrender the rights.  Therefore, you have\ncertain responsibilities if you distribute copies of the software, or if\nyou modify it: responsibilities to respect the freedom of others.\n\n  For example, if you distribute copies of such a program, whether\ngratis or for a fee, you must pass on to the recipients the same\nfreedoms that you received.  You must make sure that they, too, receive\nor can get the source code.  And you must show them these terms so they\nknow their rights.\n\n  Developers that use the GNU GPL protect your rights with two steps:\n(1) assert copyright on the software, and (2) offer you this License\ngiving you legal permission to copy, distribute and/or modify it.\n\n  For the developers' and authors' protection, the GPL clearly explains\nthat there is no warranty for this free software.  For both users' and\nauthors' sake, the GPL requires that modified versions be marked as\nchanged, so that their problems will not be attributed erroneously to\nauthors of previous versions.\n\n  Some devices are designed to deny users access to install or run\nmodified versions of the software inside them, although the manufacturer\ncan do so.  This is fundamentally incompatible with the aim of\nprotecting users' freedom to change the software.  The systematic\npattern of such abuse occurs in the area of products for individuals to\nuse, which is precisely where it is most unacceptable.  Therefore, we\nhave designed this version of the GPL to prohibit the practice for those\nproducts.  If such problems arise substantially in other domains, we\nstand ready to extend this provision to those domains in future versions\nof the GPL, as needed to protect the freedom of users.\n\n  Finally, every program is threatened constantly by software patents.\nStates should not allow patents to restrict development and use of\nsoftware on general-purpose computers, but in those that do, we wish to\navoid the special danger that patents applied to a free program could\nmake it effectively proprietary.  To prevent this, the GPL assures that\npatents cannot be used to render the program non-free.\n\n  The precise terms and conditions for copying, distribution and\nmodification follow.\n\n                       TERMS AND CONDITIONS\n\n  0. Definitions.\n\n  \"This License\" refers to version 3 of the GNU General Public License.\n\n  \"Copyright\" also means copyright-like laws that apply to other kinds of\nworks, such as semiconductor masks.\n\n  \"The Program\" refers to any copyrightable work licensed under this\nLicense.  Each licensee is addressed as \"you\".  \"Licensees\" and\n\"recipients\" may be individuals or organizations.\n\n  To \"modify\" a work means to copy from or adapt all or part of the work\nin a fashion requiring copyright permission, other than the making of an\nexact copy.  The resulting work is called a \"modified version\" of the\nearlier work or a work \"based on\" the earlier work.\n\n  A \"covered work\" means either the unmodified Program or a work based\non the Program.\n\n  To \"propagate\" a work means to do anything with it that, without\npermission, would make you directly or secondarily liable for\ninfringement under applicable copyright law, except executing it on a\ncomputer or modifying a private copy.  Propagation includes copying,\ndistribution (with or without modification), making available to the\npublic, and in some countries other activities as well.\n\n  To \"convey\" a work means any kind of propagation that enables other\nparties to make or receive copies.  Mere interaction with a user through\na computer network, with no transfer of a copy, is not conveying.\n\n  An interactive user interface displays \"Appropriate Legal Notices\"\nto the extent that it includes a convenient and prominently visible\nfeature that (1) displays an appropriate copyright notice, and (2)\ntells the user that there is no warranty for the work (except to the\nextent that warranties are provided), that licensees may convey the\nwork under this License, and how to view a copy of this License.  If\nthe interface presents a list of user commands or options, such as a\nmenu, a prominent item in the list meets this criterion.\n\n  1. Source Code.\n\n  The \"source code\" for a work means the preferred form of the work\nfor making modifications to it.  \"Object code\" means any non-source\nform of a work.\n\n  A \"Standard Interface\" means an interface that either is an official\nstandard defined by a recognized standards body, or, in the case of\ninterfaces specified for a particular programming language, one that\nis widely used among developers working in that language.\n\n  The \"System Libraries\" of an executable work include anything, other\nthan the work as a whole, that (a) is included in the normal form of\npackaging a Major Component, but which is not part of that Major\nComponent, and (b) serves only to enable use of the work with that\nMajor Component, or to implement a Standard Interface for which an\nimplementation is available to the public in source code form.  A\n\"Major Component\", in this context, means a major essential component\n(kernel, window system, and so on) of the specific operating system\n(if any) on which the executable work runs, or a compiler used to\nproduce the work, or an object code interpreter used to run it.\n\n  The \"Corresponding Source\" for a work in object code form means all\nthe source code needed to generate, install, and (for an executable\nwork) run the object code and to modify the work, including scripts to\ncontrol those activities.  However, it does not include the work's\nSystem Libraries, or general-purpose tools or generally available free\nprograms which are used unmodified in performing those activities but\nwhich are not part of the work.  For example, Corresponding Source\nincludes interface definition files associated with source files for\nthe work, and the source code for shared libraries and dynamically\nlinked subprograms that the work is specifically designed to require,\nsuch as by intimate data communication or control flow between those\nsubprograms and other parts of the work.\n\n  The Corresponding Source need not include anything that users\ncan regenerate automatically from other parts of the Corresponding\nSource.\n\n  The Corresponding Source for a work in source code form is that\nsame work.\n\n  2. Basic Permissions.\n\n  All rights granted under this License are granted for the term of\ncopyright on the Program, and are irrevocable provided the stated\nconditions are met.  This License explicitly affirms your unlimited\npermission to run the unmodified Program.  The output from running a\ncovered work is covered by this License only if the output, given its\ncontent, constitutes a covered work.  This License acknowledges your\nrights of fair use or other equivalent, as provided by copyright law.\n\n  You may make, run and propagate covered works that you do not\nconvey, without conditions so long as your license otherwise remains\nin force.  You may convey covered works to others for the sole purpose\nof having them make modifications exclusively for you, or provide you\nwith facilities for running those works, provided that you comply with\nthe terms of this License in conveying all material for which you do\nnot control copyright.  Those thus making or running the covered works\nfor you must do so exclusively on your behalf, under your direction\nand control, on terms that prohibit them from making any copies of\nyour copyrighted material outside their relationship with you.\n\n  Conveying under any other circumstances is permitted solely under\nthe conditions stated below.  Sublicensing is not allowed; section 10\nmakes it unnecessary.\n\n  3. Protecting Users' Legal Rights From Anti-Circumvention Law.\n\n  No covered work shall be deemed part of an effective technological\nmeasure under any applicable law fulfilling obligations under article\n11 of the WIPO copyright treaty adopted on 20 December 1996, or\nsimilar laws prohibiting or restricting circumvention of such\nmeasures.\n\n  When you convey a covered work, you waive any legal power to forbid\ncircumvention of technological measures to the extent such circumvention\nis effected by exercising rights under this License with respect to\nthe covered work, and you disclaim any intention to limit operation or\nmodification of the work as a means of enforcing, against the work's\nusers, your or third parties' legal rights to forbid circumvention of\ntechnological measures.\n\n  4. Conveying Verbatim Copies.\n\n  You may convey verbatim copies of the Program's source code as you\nreceive it, in any medium, provided that you conspicuously and\nappropriately publish on each copy an appropriate copyright notice;\nkeep intact all notices stating that this License and any\nnon-permissive terms added in accord with section 7 apply to the code;\nkeep intact all notices of the absence of any warranty; and give all\nrecipients a copy of this License along with the Program.\n\n  You may charge any price or no price for each copy that you convey,\nand you may offer support or warranty protection for a fee.\n\n  5. Conveying Modified Source Versions.\n\n  You may convey a work based on the Program, or the modifications to\nproduce it from the Program, in the form of source code under the\nterms of section 4, provided that you also meet all of these conditions:\n\n    a) The work must carry prominent notices stating that you modified\n    it, and giving a relevant date.\n\n    b) The work must carry prominent notices stating that it is\n    released under this License and any conditions added under section\n    7.  This requirement modifies the requirement in section 4 to\n    \"keep intact all notices\".\n\n    c) You must license the entire work, as a whole, under this\n    License to anyone who comes into possession of a copy.  This\n    License will therefore apply, along with any applicable section 7\n    additional terms, to the whole of the work, and all its parts,\n    regardless of how they are packaged.  This License gives no\n    permission to license the work in any other way, but it does not\n    invalidate such permission if you have separately received it.\n\n    d) If the work has interactive user interfaces, each must display\n    Appropriate Legal Notices; however, if the Program has interactive\n    interfaces that do not display Appropriate Legal Notices, your\n    work need not make them do so.\n\n  A compilation of a covered work with other separate and independent\nworks, which are not by their nature extensions of the covered work,\nand which are not combined with it such as to form a larger program,\nin or on a volume of a storage or distribution medium, is called an\n\"aggregate\" if the compilation and its resulting copyright are not\nused to limit the access or legal rights of the compilation's users\nbeyond what the individual works permit.  Inclusion of a covered work\nin an aggregate does not cause this License to apply to the other\nparts of the aggregate.\n\n  6. Conveying Non-Source Forms.\n\n  You may convey a covered work in object code form under the terms\nof sections 4 and 5, provided that you also convey the\nmachine-readable Corresponding Source under the terms of this License,\nin one of these ways:\n\n    a) Convey the object code in, or embodied in, a physical product\n    (including a physical distribution medium), accompanied by the\n    Corresponding Source fixed on a durable physical medium\n    customarily used for software interchange.\n\n    b) Convey the object code in, or embodied in, a physical product\n    (including a physical distribution medium), accompanied by a\n    written offer, valid for at least three years and valid for as\n    long as you offer spare parts or customer support for that product\n    model, to give anyone who possesses the object code either (1) a\n    copy of the Corresponding Source for all the software in the\n    product that is covered by this License, on a durable physical\n    medium customarily used for software interchange, for a price no\n    more than your reasonable cost of physically performing this\n    conveying of source, or (2) access to copy the\n    Corresponding Source from a network server at no charge.\n\n    c) Convey individual copies of the object code with a copy of the\n    written offer to provide the Corresponding Source.  This\n    alternative is allowed only occasionally and noncommercially, and\n    only if you received the object code with such an offer, in accord\n    with subsection 6b.\n\n    d) Convey the object code by offering access from a designated\n    place (gratis or for a charge), and offer equivalent access to the\n    Corresponding Source in the same way through the same place at no\n    further charge.  You need not require recipients to copy the\n    Corresponding Source along with the object code.  If the place to\n    copy the object code is a network server, the Corresponding Source\n    may be on a different server (operated by you or a third party)\n    that supports equivalent copying facilities, provided you maintain\n    clear directions next to the object code saying where to find the\n    Corresponding Source.  Regardless of what server hosts the\n    Corresponding Source, you remain obligated to ensure that it is\n    available for as long as needed to satisfy these requirements.\n\n    e) Convey the object code using peer-to-peer transmission, provided\n    you inform other peers where the object code and Corresponding\n    Source of the work are being offered to the general public at no\n    charge under subsection 6d.\n\n  A separable portion of the object code, whose source code is excluded\nfrom the Corresponding Source as a System Library, need not be\nincluded in conveying the object code work.\n\n  A \"User Product\" is either (1) a \"consumer product\", which means any\ntangible personal property which is normally used for personal, family,\nor household purposes, or (2) anything designed or sold for incorporation\ninto a dwelling.  In determining whether a product is a consumer product,\ndoubtful cases shall be resolved in favor of coverage.  For a particular\nproduct received by a particular user, \"normally used\" refers to a\ntypical or common use of that class of product, regardless of the status\nof the particular user or of the way in which the particular user\nactually uses, or expects or is expected to use, the product.  A product\nis a consumer product regardless of whether the product has substantial\ncommercial, industrial or non-consumer uses, unless such uses represent\nthe only significant mode of use of the product.\n\n  \"Installation Information\" for a User Product means any methods,\nprocedures, authorization keys, or other information required to install\nand execute modified versions of a covered work in that User Product from\na modified version of its Corresponding Source.  The information must\nsuffice to ensure that the continued functioning of the modified object\ncode is in no case prevented or interfered with solely because\nmodification has been made.\n\n  If you convey an object code work under this section in, or with, or\nspecifically for use in, a User Product, and the conveying occurs as\npart of a transaction in which the right of possession and use of the\nUser Product is transferred to the recipient in perpetuity or for a\nfixed term (regardless of how the transaction is characterized), the\nCorresponding Source conveyed under this section must be accompanied\nby the Installation Information.  But this requirement does not apply\nif neither you nor any third party retains the ability to install\nmodified object code on the User Product (for example, the work has\nbeen installed in ROM).\n\n  The requirement to provide Installation Information does not include a\nrequirement to continue to provide support service, warranty, or updates\nfor a work that has been modified or installed by the recipient, or for\nthe User Product in which it has been modified or installed.  Access to a\nnetwork may be denied when the modification itself materially and\nadversely affects the operation of the network or violates the rules and\nprotocols for communication across the network.\n\n  Corresponding Source conveyed, and Installation Information provided,\nin accord with this section must be in a format that is publicly\ndocumented (and with an implementation available to the public in\nsource code form), and must require no special password or key for\nunpacking, reading or copying.\n\n  7. Additional Terms.\n\n  \"Additional permissions\" are terms that supplement the terms of this\nLicense by making exceptions from one or more of its conditions.\nAdditional permissions that are applicable to the entire Program shall\nbe treated as though they were included in this License, to the extent\nthat they are valid under applicable law.  If additional permissions\napply only to part of the Program, that part may be used separately\nunder those permissions, but the entire Program remains governed by\nthis License without regard to the additional permissions.\n\n  When you convey a copy of a covered work, you may at your option\nremove any additional permissions from that copy, or from any part of\nit.  (Additional permissions may be written to require their own\nremoval in certain cases when you modify the work.)  You may place\nadditional permissions on material, added by you to a covered work,\nfor which you have or can give appropriate copyright permission.\n\n  Notwithstanding any other provision of this License, for material you\nadd to a covered work, you may (if authorized by the copyright holders of\nthat material) supplement the terms of this License with terms:\n\n    a) Disclaiming warranty or limiting liability differently from the\n    terms of sections 15 and 16 of this License; or\n\n    b) Requiring preservation of specified reasonable legal notices or\n    author attributions in that material or in the Appropriate Legal\n    Notices displayed by works containing it; or\n\n    c) Prohibiting misrepresentation of the origin of that material, or\n    requiring that modified versions of such material be marked in\n    reasonable ways as different from the original version; or\n\n    d) Limiting the use for publicity purposes of names of licensors or\n    authors of the material; or\n\n    e) Declining to grant rights under trademark law for use of some\n    trade names, trademarks, or service marks; or\n\n    f) Requiring indemnification of licensors and authors of that\n    material by anyone who conveys the material (or modified versions of\n    it) with contractual assumptions of liability to the recipient, for\n    any liability that these contractual assumptions directly impose on\n    those licensors and authors.\n\n  All other non-permissive additional terms are considered \"further\nrestrictions\" within the meaning of section 10.  If the Program as you\nreceived it, or any part of it, contains a notice stating that it is\ngoverned by this License along with a term that is a further\nrestriction, you may remove that term.  If a license document contains\na further restriction but permits relicensing or conveying under this\nLicense, you may add to a covered work material governed by the terms\nof that license document, provided that the further restriction does\nnot survive such relicensing or conveying.\n\n  If you add terms to a covered work in accord with this section, you\nmust place, in the relevant source files, a statement of the\nadditional terms that apply to those files, or a notice indicating\nwhere to find the applicable terms.\n\n  Additional terms, permissive or non-permissive, may be stated in the\nform of a separately written license, or stated as exceptions;\nthe above requirements apply either way.\n\n  8. Termination.\n\n  You may not propagate or modify a covered work except as expressly\nprovided under this License.  Any attempt otherwise to propagate or\nmodify it is void, and will automatically terminate your rights under\nthis License (including any patent licenses granted under the third\nparagraph of section 11).\n\n  However, if you cease all violation of this License, then your\nlicense from a particular copyright holder is reinstated (a)\nprovisionally, unless and until the copyright holder explicitly and\nfinally terminates your license, and (b) permanently, if the copyright\nholder fails to notify you of the violation by some reasonable means\nprior to 60 days after the cessation.\n\n  Moreover, your license from a particular copyright holder is\nreinstated permanently if the copyright holder notifies you of the\nviolation by some reasonable means, this is the first time you have\nreceived notice of violation of this License (for any work) from that\ncopyright holder, and you cure the violation prior to 30 days after\nyour receipt of the notice.\n\n  Termination of your rights under this section does not terminate the\nlicenses of parties who have received copies or rights from you under\nthis License.  If your rights have been terminated and not permanently\nreinstated, you do not qualify to receive new licenses for the same\nmaterial under section 10.\n\n  9. Acceptance Not Required for Having Copies.\n\n  You are not required to accept this License in order to receive or\nrun a copy of the Program.  Ancillary propagation of a covered work\noccurring solely as a consequence of using peer-to-peer transmission\nto receive a copy likewise does not require acceptance.  However,\nnothing other than this License grants you permission to propagate or\nmodify any covered work.  These actions infringe copyright if you do\nnot accept this License.  Therefore, by modifying or propagating a\ncovered work, you indicate your acceptance of this License to do so.\n\n  10. Automatic Licensing of Downstream Recipients.\n\n  Each time you convey a covered work, the recipient automatically\nreceives a license from the original licensors, to run, modify and\npropagate that work, subject to this License.  You are not responsible\nfor enforcing compliance by third parties with this License.\n\n  An \"entity transaction\" is a transaction transferring control of an\norganization, or substantially all assets of one, or subdividing an\norganization, or merging organizations.  If propagation of a covered\nwork results from an entity transaction, each party to that\ntransaction who receives a copy of the work also receives whatever\nlicenses to the work the party's predecessor in interest had or could\ngive under the previous paragraph, plus a right to possession of the\nCorresponding Source of the work from the predecessor in interest, if\nthe predecessor has it or can get it with reasonable efforts.\n\n  You may not impose any further restrictions on the exercise of the\nrights granted or affirmed under this License.  For example, you may\nnot impose a license fee, royalty, or other charge for exercise of\nrights granted under this License, and you may not initiate litigation\n(including a cross-claim or counterclaim in a lawsuit) alleging that\nany patent claim is infringed by making, using, selling, offering for\nsale, or importing the Program or any portion of it.\n\n  11. Patents.\n\n  A \"contributor\" is a copyright holder who authorizes use under this\nLicense of the Program or a work on which the Program is based.  The\nwork thus licensed is called the contributor's \"contributor version\".\n\n  A contributor's \"essential patent claims\" are all patent claims\nowned or controlled by the contributor, whether already acquired or\nhereafter acquired, that would be infringed by some manner, permitted\nby this License, of making, using, or selling its contributor version,\nbut do not include claims that would be infringed only as a\nconsequence of further modification of the contributor version.  For\npurposes of this definition, \"control\" includes the right to grant\npatent sublicenses in a manner consistent with the requirements of\nthis License.\n\n  Each contributor grants you a non-exclusive, worldwide, royalty-free\npatent license under the contributor's essential patent claims, to\nmake, use, sell, offer for sale, import and otherwise run, modify and\npropagate the contents of its contributor version.\n\n  In the following three paragraphs, a \"patent license\" is any express\nagreement or commitment, however denominated, not to enforce a patent\n(such as an express permission to practice a patent or covenant not to\nsue for patent infringement).  To \"grant\" such a patent license to a\nparty means to make such an agreement or commitment not to enforce a\npatent against the party.\n\n  If you convey a covered work, knowingly relying on a patent license,\nand the Corresponding Source of the work is not available for anyone\nto copy, free of charge and under the terms of this License, through a\npublicly available network server or other readily accessible means,\nthen you must either (1) cause the Corresponding Source to be so\navailable, or (2) arrange to deprive yourself of the benefit of the\npatent license for this particular work, or (3) arrange, in a manner\nconsistent with the requirements of this License, to extend the patent\nlicense to downstream recipients.  \"Knowingly relying\" means you have\nactual knowledge that, but for the patent license, your conveying the\ncovered work in a country, or your recipient's use of the covered work\nin a country, would infringe one or more identifiable patents in that\ncountry that you have reason to believe are valid.\n\n  If, pursuant to or in connection with a single transaction or\narrangement, you convey, or propagate by procuring conveyance of, a\ncovered work, and grant a patent license to some of the parties\nreceiving the covered work authorizing them to use, propagate, modify\nor convey a specific copy of the covered work, then the patent license\nyou grant is automatically extended to all recipients of the covered\nwork and works based on it.\n\n  A patent license is \"discriminatory\" if it does not include within\nthe scope of its coverage, prohibits the exercise of, or is\nconditioned on the non-exercise of one or more of the rights that are\nspecifically granted under this License.  You may not convey a covered\nwork if you are a party to an arrangement with a third party that is\nin the business of distributing software, under which you make payment\nto the third party based on the extent of your activity of conveying\nthe work, and under which the third party grants, to any of the\nparties who would receive the covered work from you, a discriminatory\npatent license (a) in connection with copies of the covered work\nconveyed by you (or copies made from those copies), or (b) primarily\nfor and in connection with specific products or compilations that\ncontain the covered work, unless you entered into that arrangement,\nor that patent license was granted, prior to 28 March 2007.\n\n  Nothing in this License shall be construed as excluding or limiting\nany implied license or other defenses to infringement that may\notherwise be available to you under applicable patent law.\n\n  12. No Surrender of Others' Freedom.\n\n  If conditions are imposed on you (whether by court order, agreement or\notherwise) that contradict the conditions of this License, they do not\nexcuse you from the conditions of this License.  If you cannot convey a\ncovered work so as to satisfy simultaneously your obligations under this\nLicense and any other pertinent obligations, then as a consequence you may\nnot convey it at all.  For example, if you agree to terms that obligate you\nto collect a royalty for further conveying from those to whom you convey\nthe Program, the only way you could satisfy both those terms and this\nLicense would be to refrain entirely from conveying the Program.\n\n  13. Use with the GNU Affero General Public License.\n\n  Notwithstanding any other provision of this License, you have\npermission to link or combine any covered work with a work licensed\nunder version 3 of the GNU Affero General Public License into a single\ncombined work, and to convey the resulting work.  The terms of this\nLicense will continue to apply to the part which is the covered work,\nbut the special requirements of the GNU Affero General Public License,\nsection 13, concerning interaction through a network will apply to the\ncombination as such.\n\n  14. Revised Versions of this License.\n\n  The Free Software Foundation may publish revised and/or new versions of\nthe GNU General Public License from time to time.  Such new versions will\nbe similar in spirit to the present version, but may differ in detail to\naddress new problems or concerns.\n\n  Each version is given a distinguishing version number.  If the\nProgram specifies that a certain numbered version of the GNU General\nPublic License \"or any later version\" applies to it, you have the\noption of following the terms and conditions either of that numbered\nversion or of any later version published by the Free Software\nFoundation.  If the Program does not specify a version number of the\nGNU General Public License, you may choose any version ever published\nby the Free Software Foundation.\n\n  If the Program specifies that a proxy can decide which future\nversions of the GNU General Public License can be used, that proxy's\npublic statement of acceptance of a version permanently authorizes you\nto choose that version for the Program.\n\n  Later license versions may give you additional or different\npermissions.  However, no additional obligations are imposed on any\nauthor or copyright holder as a result of your choosing to follow a\nlater version.\n\n  15. Disclaimer of Warranty.\n\n  THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY\nAPPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT\nHOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM \"AS IS\" WITHOUT WARRANTY\nOF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,\nTHE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR\nPURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM\nIS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF\nALL NECESSARY SERVICING, REPAIR OR CORRECTION.\n\n  16. Limitation of Liability.\n\n  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING\nWILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS\nTHE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY\nGENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE\nUSE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF\nDATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD\nPARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),\nEVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF\nSUCH DAMAGES.\n\n  17. Interpretation of Sections 15 and 16.\n\n  If the disclaimer of warranty and limitation of liability provided\nabove cannot be given local legal effect according to their terms,\nreviewing courts shall apply local law that most closely approximates\nan absolute waiver of all civil liability in connection with the\nProgram, unless a warranty or assumption of liability accompanies a\ncopy of the Program in return for a fee.\n\n                     END OF TERMS AND CONDITIONS\n\n            How to Apply These Terms to Your New Programs\n\n  If you develop a new program, and you want it to be of the greatest\npossible use to the public, the best way to achieve this is to make it\nfree software which everyone can redistribute and change under these terms.\n\n  To do so, attach the following notices to the program.  It is safest\nto attach them to the start of each source file to most effectively\nstate the exclusion of warranty; and each file should have at least\nthe \"copyright\" line and a pointer to where the full notice is found.\n\n     \n    Copyright (C)     \n\n    This program is free software: you can redistribute it and/or modify\n    it under the terms of the GNU General Public License as published by\n    the Free Software Foundation, either version 3 of the License, or\n    (at your option) any later version.\n\n    This program is distributed in the hope that it will be useful,\n    but WITHOUT ANY WARRANTY; without even the implied warranty of\n    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n    GNU General Public License for more details.\n\n    You should have received a copy of the GNU General Public License\n    along with this program.  If not, see  .\n\nAlso add information on how to contact you by electronic and paper mail.\n\n  If the program does terminal interaction, make it output a short\nnotice like this when it starts in an interactive mode:\n\n       Copyright (C)     \n    This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.\n    This is free software, and you are welcome to redistribute it\n    under certain conditions; type `show c' for details.\n\nThe hypothetical commands `show w' and `show c' should show the appropriate\nparts of the General Public License.  Of course, your program's commands\nmight be different; for a GUI interface, you would use an \"about box\".\n\n  You should also get your employer (if you work as a programmer) or school,\nif any, to sign a \"copyright disclaimer\" for the program, if necessary.\nFor more information on this, and how to apply and follow the GNU GPL, see\n .\n\n  The GNU General Public License does not permit incorporating your program\ninto proprietary programs.  If your program is a subroutine library, you\nmay consider it more useful to permit linking proprietary applications with\nthe library.  If this is what you want to do, use the GNU Lesser General\nPublic License instead of this License.  But first, please read\n .","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}